@@ -81,6 +81,7 @@ import inspect
 import logging
 import operator
 import argparse
+import subprocess
 
 # Installed libraries
 import meraki
@@ -90,6 +91,7 @@ from rich.table import Table
 
 # Local libraries
 from . import __version__
+from .__version__ import user_agent as USER_AGENT
 
 
 class Args:
@@ -1033,6 +1035,97 @@ def _build_parser(subparser: argparse.ArgumentParser, obj: object) -> None:
                 _build_parser(subparser, subobj)
 
 
+def _add_upgrade_parser(subparser: argparse.ArgumentParser) -> None:
+    """
+    Add options to the argument parser for software upgrade operations.
+
+    - subparser: The argparse.ArgumentParser subparser object where we can
+        build in the arguments.
+    """
+    # Add the 'upgrade' command to the main subparser
+    cmd = subparser.add_parser(
+        'upgrade',
+        add_help=False,
+        help='Upgrade Meraki-CLI components (no Dashboard interaction)',
+        description='Upgrade Meraki-CLI components (CLI utility, Meraki '
+        'Dashboard API Library, etc)\n    - No interaction with the Meraki '
+        'Dashboard will occur.\n    - Software upgrades happen on local '
+        'machine.',
+        formatter_class=argparse.RawTextHelpFormatter)
+    # Add an argument group for the different types of upgrades which can
+    #     be run
+    req_group = cmd.add_argument_group('Upgrade Options')
+    # Nest in an exclusive group so multiple options cannot be set, only one
+    excl_group = req_group.add_mutually_exclusive_group(required=True)
+    excl_group.add_argument(
+        '--upgrade-all',
+        dest='upgrade_all',
+        help='Upgrade Meraki-CLI and the Meraki Dashboard API Python SDK '
+        'package, dependencies will be left alone unless required',
+        action='store_true')
+    excl_group.add_argument(
+        '--upgrade-meraki-cli',
+        dest='upgrade_meraki_cli',
+        help='Upgrade only the Meraki-CLI package, dependencies will be left '
+        'alone unless required',
+        action='store_true')
+    excl_group.add_argument(
+        '--upgrade-meraki-sdk',
+        dest='upgrade_meraki_sdk',
+        help='Upgrade only the Meraki Dashboard API Python SDK package, '
+        'others will be left alone unless required',
+        action='store_true')
+    excl_group.add_argument(
+        '--upgrade-all-eager',
+        dest='upgrade_all_eager',
+        help='Upgrade Meraki-CLI and all its dependencies in an eager fashion',
+        action='store_true')
+    # Other arguments which can be added
+    msc_group = cmd.add_argument_group('Misc Arguments')
+    msc_group.add_argument(
+        '--no-confirm',
+        dest='no_confirm',
+        help='Don\'t interactively request confirmation before beginning '
+        'upgrade',
+        action='store_true')
+    msc_group.add_argument('-h', '--help',
+                           help='Show help for this command',
+                           action='help')
+
+
+def _upgrade(parsed_args: argparse.Namespace) -> None:
+    """
+    Execute a software upgrade process using the options included from the CLI
+
+    - parsed_args: Parsed arguments namespace object from argparse.
+    """
+    log.info('Beginning upgrade process')
+    # If the user did not set the no-confirm flag
+    if not parsed_args.no_confirm:
+        conf = input('Continue with upgrade process? (y/n)')
+        if conf[0].lower() != 'y':
+            log.critical('Upgrade cancelled by user')
+            sys.exit(1)
+    log.info('Upgrade continuation confirmed')
+    # Set the CLI options for each upgrade scenario
+    if parsed_args.upgrade_all:
+        packages = 'meraki-cli meraki'
+        strategy = 'only-if-needed'
+    elif parsed_args.upgrade_meraki_cli:
+        packages = 'meraki-cli'
+        strategy = 'only-if-needed'
+    elif parsed_args.upgrade_meraki_sdk:
+        packages = 'meraki'
+        strategy = 'only-if-needed'
+    elif parsed_args.upgrade_all_eager:
+        packages = 'meraki-cli'
+        strategy = 'eager'
+    opts = ('-m pip install --no-cache-dir --upgrade '
+            f'--upgrade-strategy {strategy} {packages}')
+    log.info(f'Executing Python with options: "{opts}"')
+    subprocess.check_call([sys.executable]+opts.split(' '))
+
+
 def main(argstring=None) -> None:
     """
     Primary function called from native script. Allow an argument string to
@@ -1118,6 +1211,7 @@ def main(argstring=None) -> None:
                             action='store_true')
     # This will contain the command types like "networks" or "switch"
     subparser = parser.add_subparsers(dest='type', title='Command Types')
+    _add_upgrade_parser(subparser)  # Build the parser using the API
     _build_parser(subparser, api)  # Build the parser using the API
     argcomplete.autocomplete(parser)  # Process the parser for auto-completion
     # If an argstring was passed in, we are probably being tested
@@ -1132,6 +1226,10 @@ def main(argstring=None) -> None:
     _args_from_file(args)  # Process any arguments from a file
     # Pull in the two logging systems
     log, meraki_log = _configure_logging(args)
+    # If an upgrade is being performed
+    if args.type == 'upgrade':
+        _upgrade(args)  # Punt to the upgrade function
+        sys.exit()  # And then exit the program
     # If an endpoint command (method) was not specified
     if not hasattr(args, 'method'):
         # Print help for the command type specified in the CLI
@@ -1148,10 +1246,11 @@ def main(argstring=None) -> None:
         if args.apiKey:
             log.debug('Instantiating Meraki API with defined API key')
             # Instantiate the real API object
-            api = meraki.DashboardAPI(args.apiKey, suppress_logging=True)
+            api = meraki.DashboardAPI(args.apiKey, suppress_logging=True,
+                                      caller=USER_AGENT)
         else:
             log.debug('Instantiating Meraki API with environment API key')
-            api = meraki.DashboardAPI(suppress_logging=True)
+            api = meraki.DashboardAPI(suppress_logging=True, caller=USER_AGENT)
     except meraki.exceptions.APIKeyError:  # If no API key was provided
         log.critical('Meraki API key not provided. Use the "-k" argument or '
                      'set an environment variable like: export '
